from typing import Iterator, List, Optional, Tuple


class Token:
    def __init__(self, y: int = -1, x: int = -1):
        self.y = y
        self.x = x
        self.shape: List[str] = []

    def read(self) -> None:
        """標準入力からToken情報を読み取る"""
        self.y, self.x = map(int, input()[:-1].split(" ")[1:])
        self.shape = []
        for _ in range(self.y):
            self.shape.append(input())

    def get_topleft_edge(self) -> Iterator[Tuple[Optional[int], Optional[int]]]:
        """*の座標(左上0, 0)を取得する。座標は左上から右 → 下に向かって探索する

        Returns:
            tuple: *の座標(なければNone, None)

        Yields:
            Iterator[Tuple[Optional[int], Optional[int]]]: *の座標
        """
        for y in range(self.y):
            for x in range(self.x):
                if self.shape[y][x] == "*":
                    yield y, x
        return None, None

    def get_bottomright_edge(self) -> Iterator[Tuple[Optional[int], Optional[int]]]:
        """get_topleft_edgeの右下から探索版。未使用"""
        for y in range(self.y)[::-1]:
            for x in range(self.x)[::-1]:
                if self.shape[y][x] == "*":
                    yield y, x
        return None, None


class Board:
    def __init__(self, y: int = -1, x: int = -1) -> None:
        self.y = y
        self.x = x
        self.board: List[str] = []
        self.enemy_pos: List[Tuple[int, int]] = []

    def read(self) -> None:
        """標準入力からBoard情報を読み取る"""
        self.y, self.x = map(int, input()[:-1].split(" ")[1:])
        _ = input()
        self.board = []
        for _ in range(self.y):
            self.board.append(input().split(" ")[1])


class Player:
    INT_INF = 424242

    def __init__(self, p_player_num: str) -> None:
        """Playerを初期化する

        Args:
            p_player_num (str): pPLAYER＿NUMBER (p1 or p2)
        """
        self.p = p_player_num
        self.char = "o" if self.p == "p1" else "x"
        self.enemy_char = "x" if self.char == "o" else "o"
        self.board = Board()
        self.token = Token()

    def is_enemy_char(self, cell: str) -> bool:
        """敵のマスか判定する"""
        return cell in (self.enemy_char, self.enemy_char.upper())

    def is_player_char(self, cell: str) -> bool:
        """自分のマスか判定する"""
        return cell in (self.char, self.char.upper())

    def set_cell_score(self) -> None:
        """マスごとの評価値を設定する"""
        self.scores = [[self.INT_INF] * self.board.x for _ in range(self.board.y)]
        enemies = self.get_enemy_pos_list()
        for y in range(self.board.y):
            for x in range(self.board.x):
                if self.board.board[y][x] != ".":
                    continue
                for enemy in enemies:
                    score = abs(y - enemy[0]) + abs(x - enemy[1])
                    if score < self.scores[y][x]:
                        self.scores[y][x] = score

    def get_enemy_pos_list(self) -> List[Tuple[int, int]]:
        """敵のマスリストを返す"""
        enemies: List[Tuple[int, int]] = []
        for y in range(self.board.y):
            for x in range(self.board.x):
                if self.is_enemy_char(self.board.board[y][x]):
                    enemies.append((y, x))
        return enemies

    def put_random(self) -> bool:
        """playerが出力すべき情報を出力する

        Returns:
            bool: Tokenを当てはめられる座標が見つかったらTrue, そうでないならFalse
        """
        self.set_cell_score()
        if self.put_token():
            return True

        print("0 0")
        return False

    def put_token(self) -> bool:
        """Tokenを配置する座標を標準出力に出力する

        Returns:
            bool: tokenを当てはめられたか
        """
        board = self.board
        answers: List[Tuple[int, Tuple[int, int]]] = []

        for y in range(board.y):
            for x in range(board.x):
                # Tokenを当てはめられるなら、スコアと座標をリストに追加する
                if not self.check_overflow(x, y) and not self.check_overlap(x, y):
                    answers.append((self.calc_score(x, y), (y, x)))
        if answers:
            answers.sort()
            answer = answers[0][1]
            print(f"{answer[0]} {answer[1]}")
            return True
        return False

    def calc_score(self, x: int, y: int) -> int:
        """Tokenを配置した場合のスコアを計算する"""
        score = 0
        for token_y in range(self.token.y):
            for token_x in range(self.token.x):
                if self.token.shape[token_y][token_x] != "*":
                    continue
                if self.board.board[token_y + y][token_x + x] != ".":
                    continue
                score += self.scores[token_y + y][token_x + x]
        return score

    def check_overflow(self, x: int, y: int) -> bool:
        """tokenがboardからはみ出ていないかチェックする

        Args:
            x (int): tokenを配置するx座標
            y (int): tokenを配置するy座標

        Returns:
            bool: はみ出ているならTrue
        """
        token = self.token
        board = self.board

        if ((x + token.x) > board.x) or ((y + token.y) > board.y):
            return True
        if (x < 0) or (y < 0):
            return True
        return False

    def check_overlap(self, x: int, y: int) -> bool:
        """tokenが敵のマスと重なっていないか、自分のマスと1つだけ重なっていないかを確認する

        Args:
            x (int): tokenを配置するx座標
            y (int): tokenを配置するy座標

        Returns:
            bool: 配置可能=False/配置不可=True
        """
        token = self.token
        overlap_counter = 0

        for token_y in range(token.y):
            for token_x in range(token.x):

                if self.is_enemy_char(self.board.board[y + token_y][x + token_x]):
                    return True

                if token.shape[token_y][token_x] == "*" and self.is_player_char(
                    self.board.board[y + token_y][x + token_x]
                ):
                    overlap_counter += 1

        if overlap_counter != 1:
            return True

        return False


def main() -> None:
    """fillerのmain部分"""
    _, _, p_player_num, _, _ = input().split(" ")

    player = Player(p_player_num)
    while True:
        player.board.read()
        player.token.read()
        player.put_random()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['john_filler', 'john_filler.nafuka']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['john_filler = john_filler.main:main']}

setup_kwargs = {
    'name': 'john-filler',
    'version': '0.1.1',
    'description': "John team's filler",
    'long_description': '# john_filler\n\njohn_fillerは42の課題、fillerのPython実装です。\n\n## 開発者向け情報\n\n### poetryインストール手順\n\n1. 公式のインストール手順に従いインストール。\n\n   pythonだと2系なので、python3に変更しています。\n\n   ```bash\n   curl -sSL https://raw.githubusercontent.com/python-poetry/poetry/master/install-poetry.py | python3 - \n   ```\n\n1. ~/.zshrcを編集。poetryのPATHを追加するコマンドを追記します。\n\n   ```bash\n   export PATH="/Users/{login}/Library/Python/3.9/bin:$PATH"\n   ```\n\n   ~/.zshrcの変更を今のshellに反映します。\n\n   ```bash\n   source ~/.zshrc\n   ```\n\n1. venvの保存場所の設定を変更\n\n   venvはデフォルトで ` /Users/{login}/Library/Caches/pypoetry/virtualenvs` にインストールされます。\n\n   プロジェクトの中に置きたいので、設定を行います。\n\n   ```bash\n   poetry config virtualenvs.in-project true\n   ```\n\n   設定を行うと、poetry addなどで、`{プロジェクトのディレクトリ}/.venv` に仮想環境が作られます。\n\n   `poetry shell` で仮想環境の中に入ることができます。\n\n### PyPIへのパッケージアップロード手順\n\n参考\n\n- [[Python] poetryを使用したPyPIパッケージ公開手順 - Qiita](https://qiita.com/sengoku/items/af301fe89b55706ca0c2)\n\n1. [PyPI](https://pypi.org/) or [TestPyPI](https://test.pypi.org/) にユーザ登録します。\n\n1. API Tokenを発行、poetryに登録します。\n\n1. パッケージをビルドします。\n\n   ```bash\n   poetry build\n   ```\n\n1. パッケージをアップロードします。\n\n   ```bash\n   # TestPyPI\n   poetry publish -r testpypi\n   # PyPI\n   poetry publish\n   ```\n\n   アップロードの際、バージョンが既にあるバージョンとは異なる必要があります（パッケージを削除してもNGなようです）。\n\n   このため、TestPyPIにアップロードして確認後、PyPIにアップロードが安全かと思います。\n',
    'author': 'John team',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
